﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace CsFileCopy
{
    /// <summary>
    /// 抽出結果をファイルに出力するためのダイアログです。
    /// </summary>
    public partial class ResultOutputDialog : Form
    {
        private MainWindow parent;
        private DataTable dataTable;

        /// <summary>
        /// 結果出力ダイアログを作成します。
        /// </summary>
        /// <param name="dt">抽出結果</param>
        public ResultOutputDialog(MainWindow parent, DataTable dt)
        {
            InitializeComponent();

            this.parent = parent;
            this.dataTable = dt;

            RestoreDialogSettings();
        }

        /// <summary>
        /// 結果をファイルに出力ボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void outputButton_Click(object sender, EventArgs e)
        {
            // 出力対象の項目が選択されていない場合
            if (!fileFullPathBox.Checked && !filePathBox.Checked && !fileNameBox.Checked && !updateDateBox.Checked)
            {
                MessageUtil.ShowAlert("出力項目が選択されていません。");
                return;
            }

            // データグリッドビューへの編集を終了させる
            parent.EndDataGridViewEdit();

            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    StreamWriter writer = new StreamWriter(saveFileDialog1.FileName, false, Encoding.GetEncoding("Shift_JIS"));

                    // トータル出力項目数
                    int totalOutputItemCount = 0;

                    if (fileFullPathBox.Checked)
                    {
                        totalOutputItemCount++;
                    }

                    if (filePathBox.Checked)
                    {
                        totalOutputItemCount++;
                    }

                    if (fileNameBox.Checked)
                    {
                        totalOutputItemCount++;
                    }

                    if (updateDateBox.Checked)
                    {
                        totalOutputItemCount++;
                    }

                    foreach (DataRow row in dataTable.Rows)
                    {
                        // チェックされているもののみ出力する場合
                        if (checkedRadio.Checked)
                        {
                            bool checkedItem = (bool)row[0];

                            // チェックされていない行は出力しない
                            if (!checkedItem)
                            {
                                continue;
                            }
                        }

                        int outputItemCount = 0;

                        // フルパスが出力対象
                        if (fileFullPathBox.Checked)
                        {
                            writer.Write(row[2] + "\\" + row[1]);
                            outputItemCount++;

                            if (outputItemCount < totalOutputItemCount)
                            {
                                // 区切り文字を出力
                                outputDelimiter(writer);
                            }
                        }

                        // ファイルパスが出力対象
                        if (filePathBox.Checked)
                        {
                            writer.Write(row[2]);
                            outputItemCount++;

                            if (outputItemCount < totalOutputItemCount)
                            {
                                // 区切り文字を出力
                                outputDelimiter(writer);
                            }
                        }

                        // ファイル名が出力対象
                        if (fileNameBox.Checked)
                        {
                            writer.Write(row[1]);
                            outputItemCount++;

                            if (outputItemCount < totalOutputItemCount)
                            {
                                // 区切り文字を出力
                                outputDelimiter(writer);
                            }
                        }

                        // 更新日時が出力対象
                        if (updateDateBox.Checked)
                        {
                            writer.Write(row[3]);
                        }

                        writer.WriteLine();
                    }

                    writer.Close();

                    MessageUtil.ShowInfo("結果をファイルに出力しました。");
                }
                catch (Exception ex)
                {
                    Log.GetInstance().Error(ex.Message);
                    Log.GetInstance().Error(ex.StackTrace);

                    MessageUtil.ShowAlert("ファイルの出力に失敗しました。\n" + ex.Message);
                }

                SaveDialogSettings();

                Close();
            }
        }

        /// <summary>
        /// 項目の区切り文字を出力します。
        /// </summary>
        /// <param name="writer">出力ストリーム</param>
        private void outputDelimiter(StreamWriter writer)
        {
            if (commaRadio.Checked)
            {
                writer.Write(",");
            }
            else
            {
                writer.Write("\t");
            }
        }

        /// <summary>
        /// ダイアログの設定状態を復元します。
        /// </summary>
        private void RestoreDialogSettings()
        {
            commaRadio.Checked = parent.AppSettings.CommaRadioChecked;
            tabRadio.Checked = parent.AppSettings.TabRadioChecked;
            fileFullPathBox.Checked = parent.AppSettings.FileFullPathChecked;
            filePathBox.Checked = parent.AppSettings.FilePathChecked;
            fileNameBox.Checked = parent.AppSettings.FileNameChecked;
            updateDateBox.Checked = parent.AppSettings.UpdateTimeChecked;
            allRadio.Checked = parent.AppSettings.AllRadioChecked;
            checkedRadio.Checked = parent.AppSettings.OnlyCheckedRadioChecked;
        }

        /// <summary>
        /// ダイアログの設定状態を保存します。
        /// </summary>
        private void SaveDialogSettings()
        {
            parent.AppSettings.CommaRadioChecked = commaRadio.Checked;
            parent.AppSettings.TabRadioChecked = tabRadio.Checked;
            parent.AppSettings.FileFullPathChecked = fileFullPathBox.Checked;
            parent.AppSettings.FilePathChecked = filePathBox.Checked;
            parent.AppSettings.FileNameChecked = fileNameBox.Checked;
            parent.AppSettings.UpdateTimeChecked = updateDateBox.Checked;
            parent.AppSettings.AllRadioChecked = allRadio.Checked;
            parent.AppSettings.OnlyCheckedRadioChecked = checkedRadio.Checked;
        }

        private void ResultOutputDialog_Load(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// キャンセルボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
